# put your input file in input_data/conll-format, name it input.conll
# the file must be CONLL-formatted, contain morphological, POS and dependency tags. Order and number of columns must match the provided example, see input.conll file
# 
# The processing steps are as follows:
# (1) prepare data by identifying sentiment words; output CONLL format, append column with an "S" where sentiment found
python add_sentiment.py input_data/conll-format/input.conll input_data/conll-format/S_input.conll
# (2) prepare data by seeking dependency patterns for opinion target extraction, append column to CONLL with rule_id (before sentiment column):
python opta_patterns.py input_data/conll-format/S_input.conll  > input_data/conll-format/full_input.conll
# (3) convert data to internal CRF Suite format:
cat input_data/conll-format/full_input.conll | python crffeaturebuilder.py > train_data/crf-format/train.crfsuite.txt
# (4) tag the data:
crfsuite tag -m models/opta.model temp/input.crfsuite.txt > temp/tagged_output.conll
